@@ -1,11 +1,26 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.99.2.1 2003/07/15 14:19:25 aland Exp $
+ * Version:	$Id: radius.c,v 1.99.2.3 2003/09/29 15:26:24 aland Exp $
  *
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
+ *
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ *   Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.99.2.1 2003/07/15 14:19:25 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.99.2.3 2003/09/29 15:26:24 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -162,7 +177,13 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original,
 		  u_short		total_length;
 		  int			len, allowed;
 		  int			msg_auth_offset = 0;
-		  uint8_t		data[MAX_PACKET_LEN];
+
+		  /*
+		   *	For simplicity in the following logic, we allow
+		   *	the attributes to "overflow" the 4k maximum
+		   *	RADIUS packet size, by one attribute.
+		   */
+		  uint8_t		data[MAX_PACKET_LEN + 256];
 		  
 		  /*
 		   *	Use memory on the stack, until we know how
@@ -206,6 +227,19 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original,
 				  continue;
 			  }
 
+			  /*
+			   *	Check that the packet is no more than
+			   *	4k in size, AFTER over-flowing the 4k
+			   *	boundary.  Note that the 'data'
+			   *	buffer, above, is one attribute longer
+			   *	than necessary, in order to permit
+			   *	this overflow.
+			   */
+			  if (total_length > MAX_PACKET_LEN) {
+				  librad_log("ERROR: Too many attributes for packet, result is larger than RFC maximum of 4k");
+				  return -1;
+			  }
+
 			  /*
 			   *	Do stuff for Message-Authenticator
 			   */
@@ -555,7 +589,8 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original,
 			  case PW_AUTHENTICATION_REJECT:
 			  case PW_ACCESS_CHALLENGE:
 			    if (original) {
-			      memcpy(hdr->vector, original->vector, AUTH_VECTOR_LEN);
+				    memcpy(hdr->vector, original->vector,
+					   AUTH_VECTOR_LEN);
 			    }
 			    break;
 			  }
