@@ -1,7 +1,7 @@
 /*
  * exec.c	Execute external programs.
  *
- * Version:	$Id: exec.c,v 1.33 2002/10/15 17:52:43 aland Exp $
+ * Version:	$Id: exec.c,v 1.37.2.1 2003/08/25 19:07:03 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  * Copyright 2000  Michael J. Hartwick <hartwick@hartwick.com>
  */
-static const char rcsid[] = "$Id: exec.c,v 1.33 2002/10/15 17:52:43 aland Exp $";
+static const char rcsid[] = "$Id: exec.c,v 1.37.2.1 2003/08/25 19:07:03 aland Exp $";
 
 #include "autoconf.h"
 
@@ -52,10 +52,12 @@ static const char rcsid[] = "$Id: exec.c,v 1.33 2002/10/15 17:52:43 aland Exp $"
  *	Return -1 on fork/other errors in the parent process.
  */
 int radius_exec_program(const char *cmd, REQUEST *request,
-			int exec_wait, const char **user_msg)
+			int exec_wait,
+			char *user_msg, int msg_len,
+			VALUE_PAIR *input_pairs,
+			VALUE_PAIR **output_pairs)
 {
 	VALUE_PAIR *vp;
-	static char message[256];
 	char answer[4096];
 	char *argv[256];
 	char *buf, *p;
@@ -66,6 +68,9 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 	int status;
 	int n, left, done;
 
+	if (user_msg) *user_msg = '\0';
+	if (output_pairs) *output_pairs = NULL;
+
 	/*
 	 *	Open a pipe for child/parent communication, if
 	 *	necessary.
@@ -76,6 +81,13 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 			       strerror(errno));
 			return -1;
 		}
+	} else {
+		/*
+		 *	We're not waiting, so we don't look for a
+		 *	message, or VP's.
+		 */
+		user_msg = NULL;
+		output_pairs = NULL;
 	}
 
 	/*
@@ -197,7 +209,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 		 */
 		envlen = 0;
 
-		for (vp = request->packet->vps; vp->next; vp = vp->next) {
+		for (vp = input_pairs; vp && vp->next; vp = vp->next) {
 			/*
 			 *	Hmm... maybe we shouldn't pass the
 			 *	user's password in an environment
@@ -218,9 +230,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 			envp[envlen++] = strdup(buffer);
 		}
 		envp[envlen] = NULL;
-
 		execve(argv[0], argv, envp);
-
 		radlog(L_ERR, "Exec-Program: FAILED to execute %s: %s",
 		       argv[0], strerror(errno));
 		exit(1);
@@ -301,26 +311,29 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 	 */
 	close(pd[0]);
 
+	DEBUG2("Exec-Program output: %s", answer);
+
 	/*
 	 *	Parse the output, if any.
 	 */
 	if (done) {
-		/*
-		 *	For backwards compatibility, first check
-		 *	for plain text (user_msg).
-		 */
-		vp = NULL;
-		n = userparse(answer, &vp);
-		if (vp) {
-			pairfree(&vp);
+		n = -1;
+		if (output_pairs) {
+			/*
+			 *	For backwards compatibility, first check
+			 *	for plain text (user_msg).
+			 */
+			vp = NULL;
+			n = userparse(answer, &vp);
+			if (vp) {
+				pairfree(&vp);
+			}
 		}
 
-		if (n < 0) {
+		if (n == T_INVALID) {
 			radlog(L_DBG, "Exec-Program-Wait: plaintext: %s", answer);
 			if (user_msg) {
-				strncpy(message, answer, sizeof(message));
-				message[sizeof(message) - 1] = 0;
-				*user_msg = message;
+				strNcpy(user_msg, answer, msg_len);
 			}
 		} else {
 			/*
@@ -346,15 +359,15 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 			}
 
 			radlog(L_DBG,"Exec-Program-Wait: value-pairs: %s", answer);
-			if (userparse(answer, &vp) < 0) {
+			if (userparse(answer, &vp) == T_INVALID) {
 				radlog(L_ERR, "Exec-Program-Wait: %s: unparsable reply", cmd);
 
 			} else {
 				/*
-				 *	Add the attributes to the reply.
+				 *	Tell the caller about the value
+				 *	pairs.
 				 */
-				pairmove(&request->reply->vps, &vp);
-				pairfree(&vp);
+				*output_pairs = vp;
 			}
 		} /* else the answer was a set of VP's, not a text message */
 	} /* else we didn't read anything from the child. */
@@ -372,6 +385,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 		}
 	}
 
-	radlog(L_ERR|L_CONS, "Exec-Program: Abnormal child exit");
+	radlog(L_ERR|L_CONS, "Exec-Program: Abnormal child exit: %s",
+	       strerror(errno));
 	return 1;
 }
