@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.5 2002/11/12 20:22:47 aland Exp $
+ * Version:	$Id: mainconfig.c,v 1.20.2.1 2003/07/22 18:17:27 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -67,7 +67,8 @@ static CONF_PARSER proxy_config[] = {
 	{ "synchronous",  PW_TYPE_BOOLEAN, 0, &mainconfig.proxy_synchronous, "no" },
 	{ "default_fallback", PW_TYPE_BOOLEAN, 0, &mainconfig.proxy_fallback, "no" },
 	{ "dead_time",    PW_TYPE_INTEGER, 0, &mainconfig.proxy_dead_time, Stringify(DEAD_TIME) },
-	{ "servers_per_realm",  PW_TYPE_INTEGER, 0, &mainconfig.max_proxies, Stringify(MAX_PROXIES) },
+        { "post_proxy_authorize", PW_TYPE_BOOLEAN, 0, &mainconfig.post_proxy_authorize, "yes" },
+	{ "wake_all_if_all_dead", PW_TYPE_BOOLEAN, 0, &mainconfig.wake_all_if_all_dead, "no" },
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -98,13 +99,8 @@ static CONF_PARSER server_config[] = {
 	{ "libdir",             PW_TYPE_STRING_PTR, 0, &radlib_dir,        "${prefix}/lib"},
 	{ "radacctdir",         PW_TYPE_STRING_PTR, 0, &radacct_dir,       "${logdir}/radacct" },
 	{ "hostname_lookups",   PW_TYPE_BOOLEAN,    0, &librad_dodns,      "no" },
-
-  /*
-   *	We don't allow re-defining this, as doing so will cause
-   *	all sorts of confusion.
-   */
-#if 0
-	{ "confdir",            PW_TYPE_STRING_PTR, 0, &radius_dir,        RADIUS_DIR },
+#ifdef WITH_SNMP
+	{ "snmp",   		PW_TYPE_BOOLEAN,    0, &mainconfig.do_snmp,      "no" },
 #endif
 	{ "max_request_time", PW_TYPE_INTEGER, 0, &mainconfig.max_request_time, Stringify(MAX_REQUEST_TIME) },
 	{ "cleanup_delay", PW_TYPE_INTEGER, 0, &mainconfig.cleanup_delay, Stringify(CLEANUP_DELAY) },
@@ -147,6 +143,9 @@ static int xlat_config(void *instance, REQUEST *request,
 	char *p, *value;
 	const char *start = fmt;
 
+	request = request;	/* -Wunused */
+	instance = instance;	/* -Wunused */
+
 	cp = NULL;
 	cs = NULL;
 
@@ -372,6 +371,16 @@ static int generate_realms(const char *filename)
 			} else {
 				c->ipaddr = ip_getaddr(authhost);
 			}
+
+			/* 
+			 * Double check length, just to be sure!
+			 */
+			if (strlen(authhost) >= sizeof(c->server)) {
+				radlog(L_ERR, "%s[%d]: Server name of length %d is greater than allowed: %d",
+				       filename, cf_section_lineno(cs),
+				       strlen(authhost), sizeof(c->server) - 1);
+				return -1;
+			}
 		}
 
 		/*
@@ -397,17 +406,15 @@ static int generate_realms(const char *filename)
 			} else {
 				c->acct_ipaddr = ip_getaddr(accthost);
 			}
+
+			if (strlen(accthost) >= sizeof(c->acct_server)) {
+				radlog(L_ERR, "%s[%d]: Server name of length %d is greater than allowed: %d",
+				       filename, cf_section_lineno(cs),
+				       strlen(accthost), sizeof(c->acct_server) - 1);
+				return -1;
+			}
 		}
 
-		/* 
-		 * Double check length, just to be sure!
-		 */
-		if (strlen(authhost) >= sizeof(c->server)) {
-			radlog(L_ERR, "%s[%d]: Server name of length %d is greater that allowed: %d",
-					filename, cf_section_lineno(cs),
-					strlen(authhost), sizeof(c->server) - 1);
-			return -1;
-		}
 		if (strlen(name2) >= sizeof(c->realm)) {
 			radlog(L_ERR, "%s[%d]: Realm name of length %d is greater than allowed %d",
 					filename, cf_section_lineno(cs),
@@ -416,7 +423,8 @@ static int generate_realms(const char *filename)
 		}
 		
 		strcpy(c->realm, name2);
-		strcpy(c->server, authhost);	
+                if (authhost) strcpy(c->server, authhost);
+		if (accthost) strcpy(c->acct_server, accthost);	
 
 		/*
 		 *	If one or the other of authentication/accounting
@@ -453,23 +461,22 @@ static int generate_realms(const char *filename)
 		if ((cf_section_value_find(cs, "notsuffix")) != NULL)
 			c->notrealm = 1;
 		if ((t = cf_section_value_find(cs,"ldflag")) != NULL) {
-			if ((strcmp(t,"1") == 0) ||
-			    (strcasecmp(t,"round_robin") == 0)) {
-				c->ldflag = 1;
+			static const LRAD_NAME_NUMBER ldflags[] = {
+				{ "fail_over",   0 },
+				{ "round_robin", 1 },
+				{ NULL, 0 }
+			};
+
+			c->ldflag = lrad_str2int(ldflags, t, -1);
+			if (c->ldflag == -1) {
+				radlog(L_ERR, "%s[%d]: Unknown value \"%s\" for ldflag",
+				       filename, cf_section_lineno(cs),
+				       t);
+				return -1;
 			}
-			else if ((strcmp(t,"0") == 0) ||
-                                 (strcasecmp(t,"fail_over") == 0)) {
-				c->ldflag = 0;
-			}
-			else {
-				/*  set to invalid value 
-                                 *  to be caught later
-                                 */
-				c->ldflag = -1;
-			}
-		}
-		else {
-			c->ldflag = 0;
+
+		} else {
+			c->ldflag = 0; /* non, make it fail-over */
 		}
 		c->active = TRUE;
 		c->acct_active = TRUE;
@@ -485,7 +492,49 @@ static int generate_realms(const char *filename)
 	 */
 	*tail = mainconfig.realms;
 	mainconfig.realms = my_realms;
-	
+
+	/*
+	 *  Ensure that all of the flags agree for the realms.
+	 *
+	 *	Yeah, it's O(N^2), but it's only once, and the
+	 *	maximum number of realms is small.
+	 */
+	for(c = mainconfig.realms; c != NULL; c = c->next) {
+		REALM *this;
+
+		/*
+		 *	Check that we cannot load balance to LOCAL
+		 *	realms, as that doesn't make any sense.
+		 */
+		if ((c->ldflag == 1) &&
+		    ((c->ipaddr == htonl(INADDR_NONE)) ||
+		     (c->acct_ipaddr == htonl(INADDR_NONE)))) {
+			radlog(L_ERR | L_CONS, "ERROR: Realm %s cannot be load balanced to LOCAL",
+			       c->realm);
+			exit(1);
+		}
+
+		/*
+		 *	Compare this realm to all others, to ensure
+		 *	that the configuration is consistent.
+		 */
+		for (this = c->next; this != NULL; this = this->next) {
+			if (strcasecmp(c->realm, this->realm) != 0) {
+				continue;
+			}
+
+			/*
+			 *	Same realm: Different load balancing
+			 *	flag: die.
+			 */
+			if (c->ldflag != this->ldflag) {
+				radlog(L_ERR | L_CONS, "ERROR: Inconsistent value in realm %s for load balancing 'ldflag' attribute",
+				       c->realm);
+				exit(1);
+			}
+		}
+	}
+
 	return 0;
 }
 
@@ -586,7 +635,7 @@ static int generate_clients(const char *filename)
 			int i, mask_length;
 
 			mask_length = atoi(netmask + 1);
-			if ((mask_length <= 0) || (mask_length > 32)) {
+			if ((mask_length < 0) || (mask_length > 32)) {
 				radlog(L_ERR, "%s[%d]: Invalid value '%s' for IP network mask.",
 						filename, cf_section_lineno(cs), netmask + 1);
 				return -1;
@@ -691,7 +740,8 @@ int read_mainconfig(int reload)
 	/* First read radiusd.conf */
 	DEBUG2("reread_config:  reading radiusd.conf");
 	if ((cs = read_radius_conf_file()) == NULL) {
-		if (debug_flag) {
+		if (debug_flag ||
+		    (radlog_dir == NULL)) {
 			radlog(L_ERR|L_CONS, "Errors reading radiusd.conf");
 		} else {
 			radlog(L_ERR|L_CONS, "Errors reading %s/radiusd.conf: For more information, please read the tail end of %s", radlog_dir, mainconfig.log_file);
