@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.70 2002/10/21 15:41:07 aland Exp $
+ * Version:	$Id: modules.c,v 1.76.2.1 2003/08/26 12:29:21 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,12 +17,12 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2003  The FreeRADIUS server project
  * Copyright 2000  Alan DeKok <aland@ox.org>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: modules.c,v 1.70 2002/10/21 15:41:07 aland Exp $";
+static const char rcsid[] = "$Id: modules.c,v 1.76.2.1 2003/08/26 12:29:21 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -37,6 +37,7 @@ static const char rcsid[] = "$Id: modules.c,v 1.70 2002/10/21 15:41:07 aland Exp
 #include "modcall.h"
 #include "conffile.h"
 #include "ltdl.h"
+#include "rad_assert.h"
 
 /*
  *	Internal list of all of the modules we have loaded.
@@ -77,7 +78,10 @@ const char *component_names[RLM_COMPONENT_COUNT] =
 	"post-auth"
 };
 
-static const char *subcomponent_names[RLM_COMPONENT_COUNT] =
+/*
+ *	Delete ASAP.
+ */
+static const char *old_subcomponent_names[RLM_COMPONENT_COUNT] =
 {
 	"authtype",
 	"autztype",
@@ -89,6 +93,18 @@ static const char *subcomponent_names[RLM_COMPONENT_COUNT] =
 	"post-authtype"
 };
 
+static const char *subcomponent_names[RLM_COMPONENT_COUNT] =
+{
+	"Auth-Type",
+	"Autz-Type",
+	"Pre-Acct-Type",
+	"Acct-Type",
+	"Session-Type",
+	"Pre-Proxy-Type",
+	"Post-Proxy-Type",
+	"Post-Auth-Type"
+};
+
 static void indexed_modcallable_free(indexed_modcallable **cf)
 {
 	indexed_modcallable	*c, *next;
@@ -161,46 +177,6 @@ int detach_modules(void)
 	return 0;
 }
 
-/*
- *  New Auth-Type's start at a large number, and go up from there.
- *
- *  We could do something more intelligent, but this should work almost
- *  all of the time.
- *
- * FIXME: move this to dict.c as dict_valadd() and dict_valdel()
- *        also clear value in module_list free (necessary?)
- */
-static int new_sectiontype_value(const char *name,int type)
-{
-	static int max_value = 32767;
-	DICT_VALUE *old_value, *new_value;
-	
-	/*
-	 *  Check to see if it's already defined.
-	 *  If so, return the old value.
-	 */
-	old_value = dict_valbyname(type, name);
-	if (old_value) 
-		return old_value->value; 
-	/* Look for the predefined Type value */
-	old_value = dict_valbyattr(type, 0);
-	if (!old_value) 
-		return 0;	/* something WIERD is happening */
-	
-	/* allocate a new value */
-	new_value = (DICT_VALUE *) rad_malloc(sizeof(DICT_VALUE));
-	
-	/* copy the old to the new */
-	memcpy(new_value, old_value, sizeof(DICT_VALUE));
-	old_value->next = new_value;
-	
-	/* set it up */
-	strNcpy(new_value->name, name, sizeof(new_value->name));
-	new_value->value = max_value++;
-	
-	return new_value->value;
-}
-
 /*
  *	Find a module on disk or in memory, and link to it.
  */
@@ -489,11 +465,13 @@ static int indexed_modcall(int comp, int idx, REQUEST *request)
 }
 
 /* Load a flat module list, as found inside an authtype{} block */
-static void load_subcomponent_section(CONF_SECTION *cs, int comp, const char *filename)
+static void load_subcomponent_section(CONF_SECTION *cs, int comp,
+				      const char *filename)
 {
 	int idx;
 	indexed_modcallable *subcomp;
 	modcallable *ml;
+	DICT_VALUE *dval;
 
 	static int meaningless_counter = 1;
 
@@ -506,13 +484,25 @@ static void load_subcomponent_section(CONF_SECTION *cs, int comp, const char *fi
 	 * nor checked for uniqueness, but all that could be fixed in a few
 	 * minutes, if anyone finds a real use for indexed config of
 	 * components other than auth. */
-	if (comp==RLM_COMPONENT_AUTH)
-		idx = new_sectiontype_value(cf_section_name2(cs),PW_AUTHTYPE);
-	else if (comp == RLM_COMPONENT_AUTZ)
-		idx = new_sectiontype_value(cf_section_name2(cs),PW_AUTZTYPE);
-	else
+	dval = NULL;
+	if (comp==RLM_COMPONENT_AUTH) {
+		dval = dict_valbyname(PW_AUTH_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_AUTZ) {
+		dval = dict_valbyname(PW_AUTZ_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_ACCT) {
+		dval = dict_valbyname(PW_ACCT_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_SESS) {
+		dval = dict_valbyname(PW_SESSION_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_POST_AUTH) {
+		dval = dict_valbyname(PW_POST_AUTH_TYPE, cf_section_name2(cs));
+	}
+
+	if (dval) {
+		idx = dval->value;
+	} else {
 		idx = meaningless_counter++;
-	
+	}
+
 	subcomp = new_sublist(comp, idx);
 	if (!subcomp) {
 		radlog(L_ERR|L_CONS,
@@ -550,6 +540,14 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 				continue;
 			}
 
+			/*
+			 *	Allow old names, too.
+			 */
+			if (strcmp(cf_section_name1(scs),
+				   old_subcomponent_names[comp]) == 0) {
+				load_subcomponent_section(scs, comp, filename);
+				continue;
+			}
 			modreflineno = cf_section_lineno(scs);
 		} else {
 			CONF_PAIR *cp;
@@ -560,7 +558,11 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 		this = compile_modsingle(comp, modref, filename, &modname);
 
 		if (comp == RLM_COMPONENT_AUTH) {
-			idx = new_sectiontype_value(modname, PW_AUTHTYPE);
+			DICT_VALUE *dval;
+
+			dval = dict_valbyname(PW_AUTH_TYPE, modname);
+			rad_assert(dval != NULL);
+			idx = dval->value;
 		} else {
 			/* See the comment in new_sublist() for explanation
 			 * of the special index 0 */
@@ -587,6 +589,36 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 	}
 }
 
+typedef struct section_type_value_t {
+	const char	*section;
+	const char	*typename;
+	int		attr;
+} section_type_value_t;
+
+static const section_type_value_t section_type_value[] = {
+	{ "authorize",    "Autz-Type",       PW_AUTZ_TYPE },
+	{ "authenticate", "Auth-Type",       PW_AUTH_TYPE },
+	{ "accounting",   "Acct-Type",       PW_ACCT_TYPE },
+	{ "session",      "Session-Type",    PW_SESSION_TYPE },
+	{ "post-auth",    "Post-Auth-Type",  PW_POST_AUTH_TYPE },
+	{ "preacct",      "Pre-Acct-Type",   PW_PRE_ACCT_TYPE },
+	{ "post-proxy",   "Post-Proxy-Type", PW_POST_PROXY_TYPE },
+	{ "pre-proxy",    "Pre-Proxy-Type",  PW_PRE_PROXY_TYPE },
+	{ NULL, NULL, 0 }
+};
+
+/*
+ *	Delete ASAP.
+ */
+static const section_type_value_t old_section_type_value[] = {
+	{ "authorize",    "autztype", PW_AUTZ_TYPE },
+	{ "authenticate", "authtype", PW_AUTH_TYPE },
+	{ "accounting",   "acctype", PW_ACCT_TYPE },
+	{ "session",      "sesstype", PW_SESSION_TYPE },
+	{ "post-auth",	  "post-authtype", PW_POST_AUTH_TYPE },
+	{ NULL, NULL, 0 }
+};
+
 /*
  *	Parse the module config sections, and load
  *	and call each module's init() function.
@@ -645,6 +677,117 @@ int setup_modules(void)
 		detach_modules();
 	}
 
+	/*
+	 *	Create any DICT_VALUE's for the types.  See
+	 *	'doc/configurable_failover' for examples of 'authtype'
+	 *	used to create new Auth-Type values.  In order to
+	 *	let the user create new names, we've got to look for
+	 *	those names, and create DICT_VALUE's for them.
+	 */
+	for (comp = 0; section_type_value[comp].section != NULL; comp++) {
+		const char	*name2;
+		DICT_ATTR	*dattr;
+		DICT_VALUE	*dval;
+		CONF_SECTION	*sub, *next;
+		CONF_PAIR	*cp;
+
+		/*
+		 *  Big-time YUCK
+		 */
+		static int my_value = 32767;
+
+		cs = cf_section_find(section_type_value[comp].section);
+
+		if (!cs) continue;
+
+		sub = NULL;
+		do {
+			/*
+			 *	See if there's a sub-section by that
+			 *	name.
+			 */
+			next = cf_subsection_find_next(cs, sub,
+						      section_type_value[comp].typename);
+
+			/*
+			 *	Allow some old names, too.
+			 */
+			if (!next && (comp <= 4)) {
+				
+				next = cf_subsection_find_next(cs, sub,
+							       old_section_type_value[comp].typename);
+			}
+			sub = next;
+
+			/*
+			 *	If so, look for it to define a new
+			 *	value.
+			 */
+			name2 = cf_section_name2(sub);
+			if (!name2) continue;
+
+
+			/*
+			 *	If the value already exists, don't
+			 *	create it again.
+			 */
+			dval = dict_valbyname(section_type_value[comp].attr,
+					      name2);
+			if (dval) continue;
+
+			/*
+       			 *	Find the attribute for the value.
+			 */
+			dattr = dict_attrbyvalue(section_type_value[comp].attr);
+			if (!dattr) continue;
+
+			/*
+			 *	Finally, create the new attribute.
+			 */
+			if (dict_addvalue(name2, dattr->name, my_value++) < 0) {
+				radlog(L_ERR, "%s", librad_errstr);
+				exit(1);
+			}
+		} while (sub != NULL);
+
+		/*
+		 *	Loop over the non-sub-sections, too.
+		 */
+		cp = NULL;
+		do {
+			/*
+			 *	See if there's a conf-pair by that
+			 *	name.
+			 */
+			cp = cf_pair_find_next(cs, cp, NULL);
+			if (!cp) break;
+
+
+			/*
+			 *	If the value already exists, don't
+			 *	create it again.
+			 */
+			name2 = cf_pair_attr(cp);
+			dval = dict_valbyname(section_type_value[comp].attr,
+					      name2);
+			if (dval) continue;
+
+			/*
+       			 *	Find the attribute for the value.
+			 */
+			dattr = dict_attrbyvalue(section_type_value[comp].attr);
+			if (!dattr) continue;
+
+			/*
+			 *	Finally, create the new attribute.
+			 */
+			if (dict_addvalue(name2, dattr->name, my_value++) < 0) {
+				radlog(L_ERR, "%s", librad_errstr);
+				exit(1);
+			}
+		} while (cp != NULL);
+	} /* over the sections which can have redundent sub-sections */
+
 	/*
 	 *  Look for the 'instantiate' section, which tells us
 	 *  the instantiation order of the modules, and also allows
@@ -700,6 +843,17 @@ int setup_modules(void)
  */
 int module_authorize(int autz_type, REQUEST *request)
 {
+	/*
+	 *	We have a proxied packet, and we've been told
+	 *	to NOT pass proxied packets through 'authorize'
+	 *	a second time.  So stop.
+	 */
+	if ((request->proxy != NULL &&
+	     mainconfig.post_proxy_authorize == FALSE)) {
+		DEBUG2(" authorize: Skipping authorize in post-proxy stage");
+		return RLM_MODULE_NOOP;
+	}
+
 	return indexed_modcall(RLM_COMPONENT_AUTZ, autz_type, request);
 }
 
@@ -722,9 +876,9 @@ int module_preacct(REQUEST *request)
 /*
  *	Do accounting for ALL configured sessions
  */
-int module_accounting(REQUEST *request)
+int module_accounting(int acct_type, REQUEST *request)
 {
-	return indexed_modcall(RLM_COMPONENT_ACCT, 0, request);
+	return indexed_modcall(RLM_COMPONENT_ACCT, acct_type, request);
 }
 
 /*
@@ -732,7 +886,7 @@ int module_accounting(REQUEST *request)
  *
  *	Returns: 0 == OK, 1 == double logins, 2 == multilink attempt
  */
-int module_checksimul(REQUEST *request, int maxsimul)
+int module_checksimul(int sess_type, REQUEST *request, int maxsimul)
 {
 	int rcode;
 
@@ -746,9 +900,9 @@ int module_checksimul(REQUEST *request, int maxsimul)
 	request->simul_max = maxsimul;
 	request->simul_mpp = 1;
 
-	rcode = indexed_modcall(RLM_COMPONENT_SESS, 0, request);
+	rcode = indexed_modcall(RLM_COMPONENT_SESS, sess_type, request);
 
-	if(rcode != RLM_MODULE_OK) {
+	if (rcode != RLM_MODULE_OK) {
 		/* FIXME: Good spot for a *rate-limited* warning to the log */
 		return 0;
 	}
@@ -775,8 +929,8 @@ int module_post_proxy(REQUEST *request)
 /*
  *	Do post-authentication for ALL configured sessions
  */
-int module_post_auth(REQUEST *request)
+int module_post_auth(int postauth_type, REQUEST *request)
 {
-	return indexed_modcall(RLM_COMPONENT_POST_AUTH, 0, request);
+	return indexed_modcall(RLM_COMPONENT_POST_AUTH, postauth_type, request);
 }
 
