@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.59 2002/10/29 22:55:26 aland Exp $
+ * Version:	$Id: proxy.c,v 1.65.2.1 2003/08/26 12:30:00 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.59 2002/10/29 22:55:26 aland Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.65.2.1 2003/08/26 12:30:00 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -55,28 +55,56 @@ int proxy_receive(REQUEST *request)
 {
 	VALUE_PAIR *proxypair;
 	VALUE_PAIR *replicatepair;
-	VALUE_PAIR *realmpair;
-	int replicating;
+        int rcode;
 
 	proxypair = pairfind(request->config_items, PW_PROXY_TO_REALM);
 	replicatepair = pairfind(request->config_items, PW_REPLICATE_TO_REALM);
-	if(proxypair) {
-		realmpair = proxypair;
-		replicating = 0;
-	} else if(replicatepair) {
-		realmpair = replicatepair;
-		replicating = 1;
-	} else {
+        if (proxypair) {
+            /* Don't do anything*/
+
+        } else if (replicatepair) {
+            /*
+             *  The request was replicated, so we don't process the response.
+             */
+            return RLM_MODULE_HANDLED;
+
+        } else {
 		radlog(L_PROXY, "Proxy reply to packet with no Realm");
-		return -1;
+		return RLM_MODULE_FAIL;
 	}
 
-	/*
-	 *	Don't touch the reply VP's.  Assume that a module
-	 *	takes care of that...
+        /*
+         *	Delete any reply we had accumulated until now.
 	 */
+        pairfree(&request->reply->vps);
 
-	return replicating?1:0;
+	/*
+	 *	Run the packet through the post-proxy stage,
+	 *	BEFORE playing games with the attributes.
+	 */
+        rcode = module_post_proxy(request);
+
+        /*
+         *	Delete the Proxy-State Attributes from the reply.
+         *	These include Proxy-State attributes from us and
+         *	remote server.
+	 */
+        pairdelete(&request->proxy_reply->vps, PW_PROXY_STATE);
+
+	/*
+	 *	Add the attributes left in the proxy reply to
+	 *	the reply list.
+	 */
+        pairadd(&request->reply->vps, request->proxy_reply->vps);
+        request->proxy_reply->vps = NULL;
+
+        /*
+         *	Free any other configuration items and proxy pairs
+         */
+        pairfree(&request->config_items);
+        pairfree(&request->proxy->vps);
+
+        return rcode;
 }
 
 /*
@@ -97,6 +125,96 @@ static void proxy_addinfo(REQUEST *request)
 	pairadd(&request->proxy->vps, proxy_pair);
 }
 
+
+/*
+ *	Like realm find, but does load balancing, and we don't
+ *	wake up any sleeping realms.  Someone should already have
+ *	done that.
+ *
+ *	It also does NOT do fail-over to default if the realms are dead,
+ *	as that decision has already been made.
+ */
+static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
+			      int accounting)
+{
+	REALM	*cl;
+	REALM	*array[32];	/* maximum number of load balancing realms */
+	int	size;
+
+	size = 0;
+	for (cl = mainconfig.realms; cl; cl = cl->next) {
+		/*
+		 *	Wake up any sleeping realm.
+		 *
+		 *	Note that the 'realm find' function will only
+		 *	wake up the FIRST realm which matches.  We've
+		 *	got to wake up ALL of the matching realms.
+		 */
+		if (cl->wakeup <= request->timestamp) {
+			cl->active = TRUE;
+		}
+		if (cl->acct_wakeup <= request->timestamp) {
+			cl->acct_active = TRUE;
+		}
+
+		/*
+		 *	Asked for auth/acct, and the auth/acct server
+		 *	is not active.  Skip it.
+		 */
+		if ((!accounting && !cl->active) ||
+		    (accounting && !cl->acct_active)) {
+			continue;
+		}
+
+		/*
+		 *	The realm name doesn't match, skip it.
+		 */
+		if (strcasecmp(cl->realm, realm_name) != 0) {
+			continue;
+		}
+
+		/*
+		 *	Fail-over, pick the first one that matches.
+		 */
+		if ((size == 0) && /* if size > 0, we have round-robin */
+		    (cl->ldflag == 0)) {
+			return cl;
+		}
+
+		/*
+		 *	Else we're doing round-robin.
+		 */
+		array[size] = cl;
+		size++;
+
+		/*
+		 *	If too many realms are found, then limit
+		 *	the number to 32.
+		 */
+		if (size >= 32) {
+			break;
+		}
+	}
+
+	/*
+	 *	WTF?  We didn't find a live realm!
+	 *
+	 *	This means that 'rlm_realm' found a live realm for
+	 *	us, and then it was marked dead before code execution
+	 *	got here.  The only thing to do is to dump the request.
+	 */
+	if (size == 0) {
+		return NULL;
+	}
+
+	/*
+	 *	Pick a random realm.  It's not true round-robin,
+	 *	but over a large number of requests, it statistically
+	 *	distributes the requests evenly among the realms.
+	 */
+	return array[(int) (lrad_rand() % size)];
+}
+
 /*
  *	Relay the request to a remote server.
  *	Returns:
@@ -124,10 +242,10 @@ int proxy_send(REQUEST *request)
 	 */
 	if ((request->packet->code != PW_AUTHENTICATION_REQUEST) &&
 	    (request->packet->code != PW_ACCOUNTING_REQUEST)) {
-	  return RLM_MODULE_FAIL;
+		return RLM_MODULE_FAIL;
 	}
 
-	/* 
+	/*
 	 *	The timestamp is used below to figure the
 	 *	next_try. The request needs to "hang around" until
 	 *	either the other server sends a reply or the retry
@@ -155,6 +273,19 @@ int proxy_send(REQUEST *request)
 		return RLM_MODULE_NOOP;
 	}
 
+	/*
+	 *	If the server has already decided to reject the request,
+	 *	then don't try to proxy it.
+	 */
+	if (request->reply->code == PW_AUTHENTICATION_REJECT) {
+		DEBUG2("Cancelling proxy as request was already rejected");
+		return RLM_MODULE_REJECT;
+	}
+	if (((vp = pairfind(request->config_items, PW_AUTH_TYPE)) != NULL) &&
+	    (vp->lvalue == PW_AUTHTYPE_REJECT)) {
+		DEBUG2("Cancelling proxy as request was already rejected");
+		return RLM_MODULE_REJECT;
+	}
 	/*
 	 *	Length == 0 means it exists, but there's no realm.
 	 *	Don't proxy it.
@@ -166,13 +297,11 @@ int proxy_send(REQUEST *request)
 	realmname = (char *)realmpair->strvalue;
 
 	/*
-	 *	Look for the realm, letting realm_find take care
-	 *	of the "NULL" realm.
-	 *
-	 *	If there is no such realm, then exit.
-	 *	Maybe we should log an error?
+	 *	Look for the realm, using the load balancing
+	 *	version of realm find.
 	 */
-	realm = realm_find(realmname, (request->packet->code == PW_ACCOUNTING_REQUEST));
+	realm = proxy_realm_ldb(request, realmname,
+				(request->packet->code == PW_ACCOUNTING_REQUEST));
 	if (realm == NULL) {
 		return RLM_MODULE_FAIL;
 	}
