@@ -1,19 +1,23 @@
 /*
  * rlm_dbm_cat.c :    List rlm_dbm DBM file
  *
- * Version:     $Id: rlm_dbm_cat.c,v 1.5 2002/06/21 19:57:27 cparker Exp $
+ * Version:     $Id: rlm_dbm_cat.c,v 1.6.2.1 2003/09/12 19:18:03 phampson Exp $
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Copyright 2001 Koulik Andrei, Sandy Service
+ * Copyright 2001 Koulik Andrei, Sandy Service
  */
 
 #include "autoconf.h"
@@ -65,17 +69,19 @@ static void dump_record(datum key,datum data)
 	putchar('\n');
 }
 
-static void usage(){
+static void usage(void)
+{
 	fprintf(stderr, "Usage: %s: [-f file] [-w] [-i number] [-l number] [-v]\n\n",progname);
 	
 	exit(1);
 }
+
 int main(int n, char **argv) {
 
-	char 	*fname = NULL;
+	const char	*fname = NULL;
 	DBM  	*pdb;
 	datum	k,d;
-	char 	ch;
+	int 	ch;
 	int 	i;
 	
 	progname = argv[0];
@@ -94,7 +100,7 @@ int main(int n, char **argv) {
 					break;
 			case 'f':	fname = optarg;
 					break;
-			case 'v':	printf("%s: $Id: rlm_dbm_cat.c,v 1.5 2002/06/21 19:57:27 cparker Exp $\n",progname);
+			case 'v':	printf("%s: $Id: rlm_dbm_cat.c,v 1.6.2.1 2003/09/12 19:18:03 phampson Exp $\n",progname);
 					exit(0);
 					break;
 			default : usage(); exit(1); break;
