@@ -1,7 +1,7 @@
 /*
  * rlm_example.c	
  *
- * Version:	$Id: rlm_example.c,v 1.23 2001/02/20 20:51:33 pacman Exp $
+ * Version:	$Id: rlm_example.c,v 1.24.4.1 2003/07/14 17:29:29 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_example.c,v 1.23 2001/02/20 20:51:33 pacman Exp $";
+static const char rcsid[] = "$Id: rlm_example.c,v 1.24.4.1 2003/07/14 17:29:29 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -99,6 +99,10 @@ static int example_instantiate(CONF_SECTION *conf, void **instance)
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
@@ -238,7 +242,10 @@ module_t rlm_example = {
 		example_authorize,	/* authorization */
 		example_preacct,	/* preaccounting */
 		example_accounting,	/* accounting */
-		example_checksimul	/* checksimul */
+		example_checksimul,	/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	example_detach,			/* detach */
 	NULL,				/* destroy */
