@@ -1,7 +1,7 @@
 /*
  * rlm_smb.c
  *
- * Version:	$Id: rlm_smb.c,v 1.2 2002/08/24 16:55:00 aland Exp $
+ * Version:	$Id: rlm_smb.c,v 1.2.4.1 2003/07/14 17:29:31 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 
 #include "valid.h"
 
-static const char rcsid[] = "$Id: rlm_smb.c,v 1.2 2002/08/24 16:55:00 aland Exp $";
+static const char rcsid[] = "$Id: rlm_smb.c,v 1.2.4.1 2003/07/14 17:29:31 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -83,6 +83,10 @@ static int smb_instantiate(CONF_SECTION *conf, void **instance)
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
