@@ -1,6 +1,6 @@
 /*
  * x99_util.c	
- * $Id: x99_util.c,v 1.10 2002/07/18 06:05:22 fcusack Exp $
+ * $Id: x99_util.c,v 1.11 2002/10/31 10:01:23 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include <openssl/des.h> /* des_cblock */
 
 
-static const char rcsid[] = "$Id: x99_util.c,v 1.10 2002/07/18 06:05:22 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_util.c,v 1.11 2002/10/31 10:01:23 fcusack Exp $";
 
 
 /* Card name to int mappings */
@@ -247,8 +247,11 @@ x99_get_user_info(const char *pwdfile, const char *username,
     (void) fclose(fp);
     free(p);
     if (!found) {
+#if 0
+	/* Noisy ... let the caller report this. */
 	x99_log(X99_LOG_AUTH, "x99_get_user_info: [%s] not found in %s",
 		username, pwdfile);
+#endif
 	return -1;
     }
 
